#include <stdio.h>
#include <stdlib.h>

void print_array(int* array, size_t len) {
    // Print contents of array
    printf("[ ");
    for (size_t i=0; i<len; i++) {
        printf("%d, ", array[i]);
    }
    printf("]\n");
}

// Takes ownership of array
// Returns ownership of new array to caller
int* add_to_front(int* array, size_t* len, int value) {
    // Malloc space for a new array
    int* new_array = malloc((*len+1)*sizeof(int));
    if (new_array == NULL) {
        printf("Malloc failed!\n");
        exit(2);
    }

    // Copy over current values
    for (size_t i=0; i<*len; i++) {
        new_array[i+1] = array[i];
    }

    // Add new value to front
    new_array[0] = value;

    // Free old array memory
    free(array);

    // Update array length
    *len += 1;

    // Return new array pointer
    return new_array;
}

int main(void) {
    // Create new heap array
    size_t len = 10;
    int* array = (int*)malloc(len * sizeof(int));
    if (array == NULL) {
        fprintf(stderr, "Out of memory!\n");
        return 1;
    }

    // Write some values into array
    for (int i=0; i<len; i++) {
        array[i] = 10*i;
    }

    // Print array
    printf("Original array\n");
    print_array(array, len);

    // Add item to front of array
    array = add_to_front(array, &len, 1337);

    // Print array
    printf("\n\nNew array\n");
    print_array(array, len);

    // Free array
    free(array);

    return 0;
}

