#include <stdio.h>
#include <stdlib.h>

struct node {
    int value;
    struct node* next;
};


// Returns ownership of the list to the caller
struct node* list_create(void) {
    return NULL;
}

// Takes ownership of the list
// Returns ownership of the list to the caller
struct node* list_append_end(struct node* list, int value) {

    // make a new node
    struct node* new_node = malloc(sizeof(struct node));
    if (new_node == NULL) {
        exit(1);
    }

    // put a value in it
    new_node->value = value;
    // SAME AS: (*new_node).value = value;

    // put a next pointer in it
    new_node->next = NULL;

    if (list == NULL) {
        return new_node;
    } else {
        // loop until I find the end
        struct node* curr_node = list;
        while (curr_node->next != NULL) {
            curr_node = curr_node->next;
        }
        curr_node->next = new_node;
        return list;
    }
}

// Borrows list
void list_print(struct node* list) {
    printf("List: ");
    struct node* curr_node = list;
    while (curr_node != NULL) {
        printf("[%d]->", curr_node->value);
        curr_node = curr_node->next;
    }
    printf("(null)\n");
}

// Take ownership of the list
void list_destroy(struct node* list) {
    struct node* curr_node = list;
    while (curr_node != NULL) {
        struct node* next = curr_node->next;
        free(curr_node);
        curr_node = next;
    }
}

// Borrow the list transiently
int list_get_at_index(struct node* list, size_t index) {
    struct node* curr_node = list;
    size_t pos = 0;
    while(curr_node != NULL) {
        if (pos == index) {
            return curr_node->value;
        }
        pos++;
        curr_node = curr_node->next;
    }
    return 0;
}

struct node* list_add_to_front(struct node* list, int value) {
    // make a new node
    struct node* new_node = malloc(sizeof(struct node));
    if (new_node == NULL) {
        exit(1);
    }

    // put a value in it
    new_node->value = value;
    // SAME AS: (*new_node).value = value;

    // put a next pointer in it
    new_node->next = list;

    return new_node;
}

int main(void) {

    // Create a linked list
    struct node* linked_list = list_create();

    // Append items to list
    linked_list = list_append_end(linked_list, 1);
    linked_list = list_append_end(linked_list, 3);
    linked_list = list_append_end(linked_list, 5);

    // Print list
    list_print(linked_list);


    printf("%d\n", list_get_at_index(linked_list, 1));

    linked_list = list_add_to_front(linked_list, 4);
    list_print(linked_list);

    
    list_destroy(linked_list);


    return 0;
}

